#include "common.h"
#include "com.h"
#include "kcomm.h"
#include "mainWnd.h"
#include "filesWnd.h"
#include "addrWnd.h"

HWND hMainWnd;

void Log(LPCSTR str)
{
	char* buf;
	DWORD len;

	len = GetWindowTextLength(GetDlgItem(hMainWnd, IDC_DEBUG)) + strlength(str) + 1;
	if(!(buf = halloc(len)))
		return;
	GetDlgItemText(hMainWnd, IDC_DEBUG, buf, len);
	strplus(buf, str);
	SetDlgItemText(hMainWnd, IDC_DEBUG, buf);
	hfree(buf);

	SendMessage(GetDlgItem(hMainWnd, IDC_DEBUG), EM_SETSEL, len, len);
	SendMessage(GetDlgItem(hMainWnd, IDC_DEBUG), EM_SCROLLCARET, 0, 0);

	RedrawWindow(GetDlgItem(hMainWnd, IDC_DEBUG), NULL, NULL, RDW_INVALIDATE | RDW_UPDATENOW);

#ifdef _DEBUG
	OutputDebugString(str);
#endif

	return;
}

void LogHex(DWORD data)
{
	BYTE buf[9];

	wsprintf(buf, "%02X", data);
	Log(buf);

	return;
}

void LogDump(BYTE* ptr, DWORD size)
{
	while(size--)
	{
		LogHex(*ptr++);
		Log(" ");
	}
}

BOOL getTelNo(HANDLE hcom)
{
	unsigned char buf[32], tmp[32];
	int i;
	DWORD bytes;

	if(WriteByte(hcom, 0x8E))
	{
		Log("ERROR: GTN Write failed\r\n");
		return TRUE;
	}
	for(i = 0;;)
	{
		ReadFile(hcom, buf, 1, &bytes, NULL);
		if(!bytes)
			continue;
		if(buf[0] == 0xAA)	// End of Number
			break;
		if(buf[0] == 0x9A)
			buf[0] = 0x90;
		if(0x90 <= buf[0] && buf[0] <= 0x99)
		{
			tmp[i] = buf[0] - 0x90 + '0';
			i += bytes;
		}
	}

	//Log(tmp);
	SetDlgItemText(hMainWnd, IDC_TELNO, tmp);
	Log("OK");

	return FALSE;
}

void doSomething(void)
{
	HANDLE hcom;
	char port[16];
	DWORD baud;

	GetDlgItemText(hMainWnd, IDC_PORT, port, 16);
	baud = GetDlgItemInt(hMainWnd, IDC_BAUDRATE, NULL, FALSE);

	Log("<<GetInfo>>\r\n");
	Log("COM open...");
	if(!(hcom = StartSerialTransfer(port)))
	{
		Log("failed\r\n");
		return;
	}
	Log("OK\r\n");

	// do peer to peer!!
	{
		unsigned char cmd[4];
		BYTE* buf;
		DWORD len;

		Log("GetTelNo...");
		if(getTelNo(hcom))
		{
			CloseHandle(hcom);
			return;
		}
		Log("\r\n");

		Log("ChangeMode...");
		if(ChangeMode(hcom))
		{
			CloseHandle(hcom);
			return;
		}
		Log("OK\r\n");

		Log("ChangeBaud...");
		if(ChangeBaudrate(hcom, baud))
		{
			CloseHandle(hcom);
			return;
		}
		Log("OK\r\n");

		Log("GetProfile(0000h)...");
		cmd[0] = 0x02;
		cmd[1] = 0x04;
		cmd[2] = 0x00;	// 0000h
		cmd[3] = 0x00;
		if(sendCommand(hcom, cmd, 4, FALSE))
		{
			Log("Send failed\r\n");
			CloseHandle(hcom);
			return;
		}
		if(ReadByte(hcom) != 0x06)
		{
			Log("NOT ACK\r\n");
			WriteByte(hcom, 0x04);
			CloseHandle(hcom);
			return;
		}
		if(getResult(hcom, &buf, &len))
		{
			Log("getResult failed\r\n");
			WriteByte(hcom, 0x04);
			CloseHandle(hcom);
			return;
		}
		Log("Set...");
		{
			BYTE buf2[32];
			DWORD i = 0;

			// skip types? data
			i += ((buf[0] << 8) | buf[1]) * 2 + 2 + 1;

			lstrcpyn(buf2, buf + i + 1, buf[i] + 1);
			SetDlgItemText(hMainWnd, IDC_MANUFACTURE, buf2);
			i += buf[i] + 2;
			strnncopy(buf2, buf + i + 1, buf[i] + 1);
			SetDlgItemText(hMainWnd, IDC_MODEL, buf2);
		}
		hfree(buf);
		Log("OK\r\n");
	}

	if(EndSerialTransfer(hcom))
		return;

	Log("Close connection nicely.\r\n");

	return;
}

void checkLockNo(void)
{
	BYTE buf[16];
	HANDLE hcom;

	char port[16];
	DWORD baud;

	GetDlgItemText(hMainWnd, IDC_PORT, port, 16);
	baud = GetDlgItemInt(hMainWnd, IDC_BAUDRATE, NULL, FALSE);

	Log("<<Check Lock No>>\r\n");
	Log("COM open...");
	if(!(hcom = StartSerialTransfer(port)))
	{
		Log("failed\r\n");
		return;
	}
	Log("OK\r\n");

	Log("ChangeMode...");
	if(ChangeMode(hcom))
	{
		CloseHandle(hcom);
		return;
	}
	Log("OK\r\n");

	Log("ChangeBaud...");
	if(ChangeBaudrate(hcom, baud))
	{
		CloseHandle(hcom);
		return;
	}
	Log("OK\r\n");

	Log("CheckLockNo...");
	buf[0] = 0x00;
	buf[1] = 0x21;
	GetDlgItemText(hMainWnd, IDC_LOCKNO, buf + 2, 9);
	if(sendCommand(hcom, buf, GetWindowTextLength(GetDlgItem(hMainWnd, IDC_LOCKNO)) + 2, FALSE))
	{
		Log("ERROR: CLN Send ERR\r\n");
		CloseHandle(hcom);
		return;
	}
	if(Read(hcom, buf, 1))
	{
		Log("ERROR: CLN Read ERR\r\n");
		CloseHandle(hcom);
		return;
	}
	switch(buf[0])
	{
	case 0x06:
		Log("NICE\r\n");
		break;
	case 0x18:
		Log("BAD\r\n");
		break;
	default:
		Log("UNKNOWN CODE ");
		{
			char tmp[3];
			wsprintf(tmp, "%02X", buf[0]);
			Log(tmp);
			Log("\r\n");
		}
	}

	if(EndSerialTransfer(hcom))
		return;

	Log("Close connection nicely.\r\n");
}

BOOL GetDlgItemChecked(HWND hDlg, int nIDDlgItem)
{
	if(SendMessage(GetDlgItem(hDlg, nIDDlgItem), BM_GETCHECK, 0, 0) & BST_CHECKED)
		return TRUE;
	else
		return FALSE;
}

void searchLockNo(void)
{
	BYTE buf[16];
	HANDLE hcom;

	char port[16];
	DWORD baud;
	BOOL longmode;

	DWORD number;

	DWORD starttime, endtime;

	GetDlgItemText(hMainWnd, IDC_PORT, port, 16);
	baud = GetDlgItemInt(hMainWnd, IDC_BAUDRATE, NULL, FALSE);
	longmode = GetDlgItemChecked(hMainWnd, IDC_LNLONG);

	Log("<<Search Lock No>>\r\n");
	if(longmode)
		Log("(Long mode)\r\n");
	Log("COM open...");
	if(!(hcom = StartSerialTransfer(port)))
	{
		Log("failed\r\n");
		return;
	}
	Log("OK\r\n");

	Log("ChangeMode...");
	if(ChangeMode(hcom))
	{
		CloseHandle(hcom);
		return;
	}
	Log("OK\r\n");

	Log("ChangeBaud...");
	if(ChangeBaudrate(hcom, baud))
	{
		CloseHandle(hcom);
		return;
	}
	Log("OK\r\n");

	starttime = GetTickCount();

	// ĂlX^[g
	number = GetDlgItemInt(hMainWnd, IDC_LOCKNO, NULL, FALSE);

	Log("Searching\r\n");
	if(!longmode)
	{
		for(; number <= 9999; number++)
		{
			if(number % 100 == 0)
			{
				wsprintf(buf + 3, "%04d", number);
				SetDlgItemText(hMainWnd, IDC_LOCKNO, buf + 3);
				RedrawWindow(GetDlgItem(hMainWnd, IDC_LOCKNO), NULL, NULL, RDW_INVALIDATE | RDW_UPDATENOW);
				if(GetAsyncKeyState(VK_ESCAPE) & 0x8000)
				{
					number = 10001;
					break;
				}
			}

			buf[0] = 0x00;
			buf[1] = 0x21;
			wsprintf(buf + 2, "%04d", number);

			if(sendCommand(hcom, buf, 2 + 4, FALSE))
			{
				Log("ERROR: CLN Send ERR\r\n");
				CloseHandle(hcom);
				return;
			}
			if(Read(hcom, buf, 1))
			{
				Log("ERROR: CLN Read ERR\r\n");
				CloseHandle(hcom);
				return;
			}
			if(buf[0] == 0x06)
				break;		// YES!!
			if(buf[0] == 0x18)
				continue;	// Bad

			Log("UNKNOWN CODE ");
			{
				char tmp[3];
				wsprintf(tmp, "%02X", buf[0]);
				Log(tmp);
				Log("\r\n");
			}
		}

		endtime = GetTickCount();

		// Translate
		if(number == 10000)
			number = 100000000;
		if(number == 10001)
			number = 100000001;
	}else
	{	// LongMode
		Log("(Longmode: slow)\r\n");

		for(; number <= 99999999; number++)
		{
			if(number % 1000 == 0)
			{
				wsprintf(buf + 3, "%04d", number);
				SetDlgItemText(hMainWnd, IDC_LOCKNO, buf + 3);
				RedrawWindow(GetDlgItem(hMainWnd, IDC_LOCKNO), NULL, NULL, RDW_INVALIDATE | RDW_UPDATENOW);
				if(GetAsyncKeyState(VK_ESCAPE) & 0x8000)
				{
					number = 100000001;
					break;
				}
			}

			buf[0] = 0x00;
			buf[1] = 0x21;
			wsprintf(buf + 2, "%04d", number);

			if(sendCommand(hcom, buf, 2 + strlength(buf + 3), FALSE))
			{
				Log("ERROR: CLN Send ERR\r\n");
				CloseHandle(hcom);
				return;
			}
			if(Read(hcom, buf, 1))
			{
				Log("ERROR: CLN Read ERR\r\n");
				CloseHandle(hcom);
				return;
			}
			if(buf[0] == 0x06)
				break;		// YES!!
			if(buf[0] == 0x18)
				continue;	// Bad

			Log("UNKNOWN CODE ");
			{
				char tmp[3];
				wsprintf(tmp, "%02X", buf[0]);
				Log(tmp);
				Log("\r\n");
			}
		}
		endtime = GetTickCount();
	}

	if(number <= 99999999)
	{
		Log("Found: YOU GOT IT!\r\n");
		SetDlgItemInt(hMainWnd, IDC_LOCKNO, number, FALSE);
	}
	if(number == 100000000)
	{
		SetDlgItemText(hMainWnd, IDC_LOCKNO, "");
		if(longmode)
			Log("Not found: what happened..?\r\n");
		else
			Log("Not found: try longmode please.\r\n");
	}
	if(number == 100000001)
		Log("Stop: Escaped by user.\r\n");

	{
		char msg[80];
		wsprintf(msg, "Elapsed time: %u ms.\r\n", endtime - starttime);
		Log(msg);
	}

	if(EndSerialTransfer(hcom))
		return;

	Log("Close connection nicely.\r\n");
}

void putFile(char* name, char* fn)
{
	HANDLE hf = NULL;
	BYTE* data = NULL;
	DWORD dsize;
	DWORD read;
	BYTE* cmd = NULL;

	HANDLE hcom = NULL;

	Log("<<PutFile>>\r\n");

	Log("OpenSrc...");

	if((hf = CreateFile(fn, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, 0, NULL)) == INVALID_HANDLE_VALUE)
		goto error;

	Log("OK\r\n");
	Log("Reading...");

	dsize = GetFileSize(hf, NULL);

	if(!dsize)
	{
		Log("Can't send size 0\r\n");
		goto error;
	}

	if(!(data = halloc(dsize)))
		goto error;

	if(!ReadFile(hf, data, dsize, &read, NULL))
		goto error;

	CloseHandle(hf), hf = NULL;

	Log("OK\r\n");
	Log("Sending...");

	if(beginComm(&hcom))
		goto error;

	Log("SetFile...");

	if(!(cmd = halloc(2 + strlength(name) + 1 + 4)))
		goto error;

	cmd[0] = 0x01;
	cmd[1] = 0x10;
	strcopy(cmd + 2, name);
	cmd[2 + strlength(name) + 1 + 0] = (BYTE)((dsize >> 24) & 0xFF);
	cmd[2 + strlength(name) + 1 + 1] = (BYTE)((dsize >> 16) & 0xFF);
	cmd[2 + strlength(name) + 1 + 2] = (BYTE)((dsize >>  8) & 0xFF);
	cmd[2 + strlength(name) + 1 + 3] = (BYTE)((dsize      ) & 0xFF);

	if(sendCommand(hcom, cmd, 2 + strlength(name) + 1 + 4, FALSE))
		goto error;
	if(ReadByte(hcom) != 0x06)
	{
		Log("NAK");
		goto error;
	}

	hfree(cmd), cmd = NULL;

	Log("data...");

	if(sendCommand(hcom, data, dsize, TRUE))
		goto error;
	Log("last");
	if(ReadByte(hcom) != 0x06)
		goto error;

	Log("\r\n");
	Log("Closing...");

	if(endComm(hcom))
		goto error;
	hcom = NULL;

	Log("OK\r\n");

	hfree(data);

	return;
error:
	Log("failed\r\n");
	if(hf)
		CloseHandle(hf);
	if(data)
		hfree(data);
	if(cmd)
		hfree(cmd);
	if(hcom)
		if(endComm(hcom))
			CloseHandle(hcom);
	return;
}

// Handler //
BOOL CALLBACK MainWndProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
	switch(msg)
	{
	case WM_INITDIALOG:
		hMainWnd = hWnd;

		SetWindowText(hWnd, "KMprototype 4 au Keitai");
		{
			HICON hi = LoadImage(GetModuleHandle(NULL), MAKEINTRESOURCE(IDI_MAINICON), IMAGE_ICON, 16, 16, LR_DEFAULTCOLOR);
			SendMessage(hWnd, WM_SETICON, ICON_SMALL, (LPARAM)hi);
			DestroyIcon(hi);
		}
		{
			HICON hi = LoadImage(GetModuleHandle(NULL), MAKEINTRESOURCE(IDI_MAINICON), IMAGE_ICON, 32, 32, LR_DEFAULTCOLOR);
			SendMessage(hWnd, WM_SETICON, ICON_BIG, (LPARAM)hi);
			DestroyIcon(hi);
		}

		SetDlgItemText(hWnd, IDC_PORT, "COM5");
		SetDlgItemInt(hWnd, IDC_BAUDRATE, /*115200*/230400, FALSE);

		return TRUE;	// set default focus
	case WM_CLOSE:
		EndDialog(hWnd, 0);
		break;
	case WM_LBUTTONDOWN:
		SendMessage(hWnd, WM_NCLBUTTONDOWN, HTCAPTION, 0);
		break;
	case WM_COMMAND:
		switch(HIWORD(wParam))
		{
		case 0:
			switch(LOWORD(wParam))
			{
			case IDOK:
				PostMessage(hWnd, WM_CLOSE, 0, 0);
				break;
			case IDC_RUN:
				doSomething();
				break;
			case IDC_LNCHECK:
				checkLockNo();
				break;
			case IDC_LNSEARCH:
				searchLockNo();
				break;
			case IDC_FILES:
				if(hFilesWnd)
					SetActiveWindow(hFilesWnd);
				else
					ShowWindow(hFilesWnd = CreateDialog(GetModuleHandle(NULL), MAKEINTRESOURCE(IDD_FILESDIALOG), hMainWnd, FilesWndProc), SW_SHOW);
				break;
			case IDC_PUTFILE:
				{
					OPENFILENAME ofn = {0};
					char fname[MAX_PATH];
					char name[MAX_PATH];

					ofn.lStructSize = sizeof(ofn);
					ofn.Flags = OFN_FILEMUSTEXIST;
					ofn.hInstance = GetModuleHandle(NULL);
					ofn.hwndOwner = hWnd;
					fname[0] = '\0';
					ofn.lpstrFile = fname;
					ofn.nMaxFile = sizeof(fname);
					ofn.lpstrFileTitle = name;
					ofn.nMaxFileTitle = sizeof(name);
					ofn.lpstrFilter = "All files(*.*)\0*.*\0";
					ofn.lpstrTitle = "Send file";

					if(GetOpenFileName(&ofn))
						putFile(name, fname);
				}
				break;
			case IDC_ADDRESS:
				if(hAddrWnd)
					SetActiveWindow(hAddrWnd);
				else
					ShowWindow(hAddrWnd = CreateDialog(GetModuleHandle(NULL), MAKEINTRESOURCE(IDD_ADDRESSDIALOG), hMainWnd, AddrWndProc), SW_SHOW);
				break;
			case IDC_DEBUGCLEAR:
				SetDlgItemText(hWnd, IDC_DEBUG, "");
				break;
			case IDC_APPICON:
				{
					char name[MAX_PATH + 1];
					char* vi;
					DWORD size;

					GetModuleFileName(NULL, name, MAX_PATH);
					size = GetFileVersionInfoSize(name, 0);
					if(vi = halloc(size))
					{
						GetFileVersionInfo(name, 0, size, vi);
						{
							VS_FIXEDFILEINFO* ver = (VS_FIXEDFILEINFO*)(vi + 40);

							// nameg
							wsprintf(name, "KM Version %d.%d.%d.%d", HIWORD(ver->dwFileVersionMS), LOWORD(ver->dwFileVersionMS),
																HIWORD(ver->dwFileVersionLS), LOWORD(ver->dwFileVersionLS));
							MessageBox(hWnd, name, "About KM", MB_ICONASTERISK | MB_OK);
						}
						hfree(vi);
					}
				}
				break;
			}
			break;
		}
		break;
	}

	return FALSE;
}
